@@ -37,11 +37,12 @@ $set 3 #Common
 4 Compiled options
 5 Compiler
 6 Compiler version
-7     init
-8     keys
-9      nls
-10     menu
-11    style
+7        init
+8        keys
+9         nls
+10        menu
+20  windowmenu
+11       style
 12 Defaults
 13 disabled
 14 Error
@@ -259,10 +260,12 @@ $set 19 #main
 10 Logging to
 11 error: '-rc' requires an argument
 12 error: '-screen' requires an argument
-13 Fluxbox %s: (c) %s Henrik Kinnunen\n\
+13 Fluxbox %s: (c) %s Fluxbox Team\n\
 Website: http://www.fluxbox.org/\n\n\
 -display <string>\t\tuse display connection.\n\
 -screen <all|int,int,int>\trun on specified screens only.\n\
+-no-slit\t\t\tdo not provide a slit.\n\
+-no-toolbar\t\t\tdo not provide a toolbar.\n\
 -rc <string>\t\t\tuse alternate resource file.\n\
 -version\t\t\tdisplay version and exit.\n\
 -info\t\t\t\tdisplay some useful information.\n\
