@@ -44,18 +44,8 @@
 #include <iostream>
 #include <algorithm>
 #include <new>
-
-#ifdef HAVE_CSTRING
-  #include <cstring>
-#else
-  #include <string.h>
-#endif
-
-#ifdef HAVE_CSTDLIB
-  #include <cstdlib>
-#else
-  #include <stdlib.h>
-#endif
+#include <cstring>
+#include <cstdlib>
 
 
 using std::cerr;
@@ -831,17 +821,15 @@ void Ewmh::updateWorkspaceNames(BScreen &screen) {
     const BScreen::WorkspaceNames &workspacenames = screen.getWorkspaceNames();
     const size_t number_of_desks = workspacenames.size();
 
-    char** names = new char*[number_of_desks];
+    const char** names = new const char*[number_of_desks];
 
     for (size_t i = 0; i < number_of_desks; i++) {
-        names[i] = new char[workspacenames[i].size() + 1]; // +1 for \0
-        memset(names[i], 0, workspacenames[i].size());
-        strcpy(names[i], workspacenames[i].c_str());
+        names[i] = workspacenames[i].c_str();
     }
 
 #ifdef X_HAVE_UTF8_STRING
     int code = Xutf8TextListToTextProperty(FbTk::App::instance()->display(),
-                                names, number_of_desks, XUTF8StringStyle, &text);
+                                const_cast<char**>(names), number_of_desks, XUTF8StringStyle, &text);
     if (code != XNoMemory && code != XLocaleNotSupported) {
         XSetTextProperty(FbTk::App::instance()->display(),
                          screen.rootWindow().window(),
@@ -858,11 +846,7 @@ void Ewmh::updateWorkspaceNames(BScreen &screen) {
     }
 #endif
 
-    for (size_t i = 0; i < number_of_desks; i++)
-        delete[] names[i];
-
     delete[] names;
-
 }
 
 void Ewmh::updateCurrentWorkspace(BScreen &screen) {
@@ -1177,14 +1161,14 @@ bool Ewmh::checkClientMessage(const XClientMessageEvent &ce,
         if (winclient == 0 || winclient->fbwindow() == 0)
             return true;
         // ce.data.l[0] = gravity and flags
-        int x = (ce.data.l[0] & 0x0100) ? ce.data.l[1] :
+        int x = (ce.data.l[0] & (1 << 8)) ? ce.data.l[1] :
             winclient->fbwindow()->x();
-        int y = (ce.data.l[0] & 0x0200) ? ce.data.l[2] :
+        int y = (ce.data.l[0] & (1 << 9)) ? ce.data.l[2] :
             winclient->fbwindow()->y();
-        unsigned int width = (ce.data.l[0] & 0x0400) ? ce.data.l[3] :
-            winclient->fbwindow()->width();
-        unsigned int height = (ce.data.l[0] & 0x0800) ? ce.data.l[4] :
-            winclient->fbwindow()->height();
+        unsigned int width = (ce.data.l[0] & (1 << 10)) ? ce.data.l[3] :
+            winclient->width();
+        unsigned int height = (ce.data.l[0] & (1 << 11)) ? ce.data.l[4] :
+            winclient->height();
         int win_gravity=ce.data.l[0] & 0xFF;
         winclient->fbwindow()->moveResizeForClient(x, y, width, height,
             win_gravity, winclient->old_bw);
