@@ -40,11 +40,6 @@
 #include "FbTk/AutoReloadHelper.hh"
 #include "FbTk/STLUtil.hh"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif // HAVE_CONFIG_H
-
-
 #ifdef HAVE_CCTYPE
   #include <cctype>
 #else
@@ -360,9 +355,9 @@ void Keys::loadDefaults() {
 
 bool Keys::addBinding(const string &linebuffer) {
 
-    vector<string> val;
     // Parse arguments
-    FbTk::StringUtil::stringtok(val, linebuffer.c_str());
+    vector<string> val;
+    FbTk::StringUtil::stringtok(val, linebuffer);
 
     // must have at least 1 argument
     if (val.empty())
@@ -495,7 +490,6 @@ bool Keys::addBinding(const string &linebuffer) {
                 }
                 mod = 0;
                 key = 0;
-                type = 0;
                 context = 0;
                 isdouble = false;
             }
@@ -583,8 +577,9 @@ bool Keys::doAction(int type, unsigned int mods, unsigned int key,
 
     // if focus changes, windows will get NotifyWhileGrabbed,
     // which they tend to ignore
-    if (type == KeyPress)
+    if (type == KeyPress) {
         XUngrabKeyboard(Fluxbox::instance()->display(), CurrentTime);
+    }
 
     WinClient *old = WindowCmd<void>::client();
     WindowCmd<void>::setClient(current);
