@@ -27,12 +27,14 @@
 #include "Window.hh"
 #include "WinClient.hh"
 #include "FbMenu.hh"
+#include "MenuCreator.hh"
 #include "FbCommands.hh"
 #include "fluxbox.hh"
 #include "Layer.hh"
 #include "Debug.hh"
 
 #include "FbTk/I18n.hh"
+#include "FbTk/FbString.hh"
 #include "FbTk/StringUtil.hh"
 #include "FbTk/FileUtil.hh"
 #include "FbTk/MenuItem.hh"
@@ -43,17 +45,7 @@
 #include "FbTk/RefCount.hh"
 #include "FbTk/Util.hh"
 
-#ifdef HAVE_CSTRING
-  #include <cstring>
-#else
-  #include <string.h>
-#endif
-
-//use GNU extensions
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE
-#endif // _GNU_SOURCE
-
+#include <cstring>
 #include <set>
 
 
@@ -68,6 +60,26 @@ using std::ofstream;
 using std::hex;
 using std::dec;
 
+using FbTk::StringUtil::getStringBetween;
+using FbTk::StringUtil::removeFirstWhitespace;
+using FbTk::StringUtil::removeTrailingWhitespace;
+using FbTk::StringUtil::toLower;
+using FbTk::StringUtil::toLower;
+using FbTk::StringUtil::extractNumber;
+using FbTk::StringUtil::expandFilename;
+
+namespace {
+
+inline bool isComment(std::string& line) {
+    removeFirstWhitespace(line);
+    removeTrailingWhitespace(line);
+    if (line.size() == 0 || line[0] == '#')
+        return true;
+    return false;
+}
+
+}
+
 /*------------------------------------------------------------------*\
 \*------------------------------------------------------------------*/
 
@@ -256,6 +268,7 @@ string escapeRememberChars(const string& str) {
                 escaped_str += '\\';
             default:
                 escaped_str += *i;
+                break;
         }
     }
 
@@ -264,8 +277,8 @@ string escapeRememberChars(const string& str) {
 
 class RememberMenuItem : public FbTk::MenuItem {
 public:
-    RememberMenuItem(const FbTk::FbString &label,
-                     Remember::Attribute attrib) :
+    RememberMenuItem(const FbTk::BiDiString &label,
+                     const Remember::Attribute attrib) :
         FbTk::MenuItem(label),
         m_attrib(attrib) {
         setToggleItem(true);
@@ -278,8 +291,7 @@ public:
 
         if (FbMenu::window()->numClients()) // ensure it HAS clients
             return Remember::instance().isRemembered(FbMenu::window()->winClient(), m_attrib);
-        else
-            return false;
+        return false;
     }
 
     bool isEnabled() const {
@@ -290,21 +302,22 @@ public:
             return true;
         else if (FbMenu::window()->numClients())
             return (Remember::instance().isRemembered(FbMenu::window()->winClient(), Remember::REM_WORKSPACE));
-        else
-            return false;
+        return false;
     }
 
     void click(int button, int time, unsigned int mods) {
         // reconfigure only does stuff if the apps file has changed
-        Remember::instance().checkReload();
+        Remember& r = Remember::instance();
+        r.checkReload();
         if (FbMenu::window() != 0) {
+            WinClient& wc = FbMenu::window()->winClient();
             if (isSelected()) {
-                Remember::instance().forgetAttrib(FbMenu::window()->winClient(), m_attrib);
+                r.forgetAttrib(wc, m_attrib);
             } else {
-                Remember::instance().rememberAttrib(FbMenu::window()->winClient(), m_attrib);
+                r.rememberAttrib(wc, m_attrib);
             }
         }
-        Remember::instance().save();
+        r.save();
         FbTk::MenuItem::click(button, time, mods);
     }
 
@@ -313,51 +326,37 @@ private:
 };
 
 FbTk::Menu *createRememberMenu(BScreen &screen) {
+
+    _FB_USES_NLS;
+
+    static const struct { bool is_alpha; const FbTk::BiDiString label; Remember::Attribute attr; } _entries[] = {
+        { false, _FB_XTEXT(Remember, Workspace, "Workspace", "Remember Workspace"), Remember::REM_WORKSPACE },
+        { false, _FB_XTEXT(Remember, JumpToWorkspace, "Jump to workspace", "Change active workspace to remembered one on open"), Remember::REM_JUMPWORKSPACE },
+        { false, _FB_XTEXT(Remember, Head, "Head", "Remember Head"), Remember::REM_HEAD},
+        { false, _FB_XTEXT(Remember, Dimensions, "Dimensions", "Remember Dimensions - with width and height"), Remember::REM_DIMENSIONS},
+        { false, _FB_XTEXT(Remember, Position, "Position", "Remember position - window co-ordinates"), Remember::REM_POSITION},
+        { false, _FB_XTEXT(Remember, Sticky, "Sticky", "Remember Sticky"), Remember::REM_STUCKSTATE},
+        { false, _FB_XTEXT(Remember, Decorations, "Decorations", "Remember window decorations"), Remember::REM_DECOSTATE},
+        { false, _FB_XTEXT(Remember, Shaded, "Shaded", "Remember shaded"), Remember::REM_SHADEDSTATE},
+        { false, _FB_XTEXT(Remember, Minimized, "Minimized", "Remember minimized"), Remember::REM_MINIMIZEDSTATE},
+        { false, _FB_XTEXT(Remember, Maximized, "Maximized", "Remember maximized"), Remember::REM_MAXIMIZEDSTATE},
+        { false, _FB_XTEXT(Remember, Fullscreen, "Fullscreen", "Remember fullscreen"), Remember::REM_FULLSCREENSTATE},
+        { true,  _FB_XTEXT(Remember, Alpha, "Transparency", "Remember window tranparency settings"), Remember::REM_ALPHA},
+        { false, _FB_XTEXT(Remember, Layer, "Layer", "Remember Layer"), Remember::REM_LAYER},
+        { false, _FB_XTEXT(Remember, SaveOnClose, "Save on close", "Save remembered attributes on close"), Remember::REM_SAVEONCLOSE}
+    };
+    bool needs_alpha = (FbTk::Transparent::haveComposite() || FbTk::Transparent::haveRender());
+
     // each fluxboxwindow has its own windowmenu
     // so we also create a remember menu just for it...
-    FbTk::Menu *menu = screen.createMenu("Remember");
-
-    // if enabled, then we want this to be a unavailable menu
-    /*
-    if (!enabled) {
-        FbTk::MenuItem *item = new FbTk::MenuItem("unavailable");
-        item->setEnabled(false);
-        menu->insert(item);
-        menu->updateMenu();
-        return menu;
+    FbTk::Menu *menu = MenuCreator::createMenu("Remember", screen);
+    size_t i;
+    for (i = 0; i < sizeof(_entries)/sizeof(_entries[0]); i++) {
+        if (_entries[i].is_alpha && !needs_alpha) { // skip alpha-entry when not needed
+            continue;
+        }
+        menu->insertItem(new RememberMenuItem(_entries[i].label, _entries[i].attr));
     }
-    */
-    _FB_USES_NLS;
-    menu->insert(new RememberMenuItem(_FB_XTEXT(Remember, Workspace, "Workspace", "Remember Workspace"),
-                                      Remember::REM_WORKSPACE));
-    menu->insert(new RememberMenuItem(_FB_XTEXT(Remember, JumpToWorkspace, "Jump to workspace", "Change active workspace to remembered one on open"),
-                                      Remember::REM_JUMPWORKSPACE));
-    menu->insert(new RememberMenuItem(_FB_XTEXT(Remember, Head, "Head", "Remember Head"),
-                                      Remember::REM_HEAD));
-    menu->insert(new RememberMenuItem(_FB_XTEXT(Remember, Dimensions, "Dimensions", "Remember Dimensions - with width and height"),
-                                      Remember::REM_DIMENSIONS));
-    menu->insert(new RememberMenuItem(_FB_XTEXT(Remember, Position, "Position", "Remember position - window co-ordinates"),
-                                      Remember::REM_POSITION));
-    menu->insert(new RememberMenuItem(_FB_XTEXT(Remember, Sticky, "Sticky", "Remember Sticky"),
-                                      Remember::REM_STUCKSTATE));
-    menu->insert(new RememberMenuItem(_FB_XTEXT(Remember, Decorations, "Decorations", "Remember window decorations"),
-                                      Remember::REM_DECOSTATE));
-    menu->insert(new RememberMenuItem(_FB_XTEXT(Remember, Shaded, "Shaded", "Remember shaded"),
-                                      Remember::REM_SHADEDSTATE));
-    menu->insert(new RememberMenuItem(_FB_XTEXT(Remember, Minimized, "Minimized", "Remember minimized"),
-                                      Remember::REM_MINIMIZEDSTATE));
-    menu->insert(new RememberMenuItem(_FB_XTEXT(Remember, Maximized, "Maximized", "Remember maximized"),
-                                      Remember::REM_MAXIMIZEDSTATE));
-    menu->insert(new RememberMenuItem(_FB_XTEXT(Remember, Fullscreen, "Fullscreen", "Remember fullscreen"),
-                                      Remember::REM_FULLSCREENSTATE));
-    if (FbTk::Transparent::haveComposite()
-        || FbTk::Transparent::haveRender())
-        menu->insert(new RememberMenuItem(_FB_XTEXT(Remember, Alpha, "Transparency", "Remember window tranparency settings"),
-                                          Remember::REM_ALPHA));
-    menu->insert(new RememberMenuItem(_FB_XTEXT(Remember, Layer, "Layer", "Remember Layer"),
-                                      Remember::REM_LAYER));
-    menu->insert(new RememberMenuItem(_FB_XTEXT(Remember, SaveOnClose, "Save on close", "Save remembered attributes on close"),
-                                      Remember::REM_SAVEONCLOSE));
 
     menu->updateMenu();
     return menu;
@@ -366,15 +365,15 @@ FbTk::Menu *createRememberMenu(BScreen &screen) {
 // offset is the offset in the string that we start looking from
 // return true if all ok, false on error
 bool handleStartupItem(const string &line, int offset) {
+
+    Fluxbox* fb = Fluxbox::instance();
+    unsigned int screen = fb->keyScreen()->screenNumber();
     int next = 0;
     string str;
-    unsigned int screen = Fluxbox::instance()->keyScreen()->screenNumber();
 
     // accept some options, for now only "screen=NN"
     // these option are given in parentheses before the command
-    next = FbTk::StringUtil::getStringBetween(str,
-                                              line.c_str() + offset,
-                                              '(', ')');
+    next = getStringBetween(str, line.c_str() + offset, '(', ')');
     if (next > 0) {
         // there are some options
         string option;
@@ -383,7 +382,7 @@ bool handleStartupItem(const string &line, int offset) {
         if (pos > 0) {
             option = str.substr(0, pos);
             if (strcasecmp(option.c_str(), "screen") == 0) {
-                error = !FbTk::StringUtil::extractNumber(str.c_str() + pos + 1, screen);
+                error = !extractNumber(str.c_str() + pos + 1, screen);
             } else {
                 error = true;
             }
@@ -398,9 +397,7 @@ bool handleStartupItem(const string &line, int offset) {
         next = 0;
     }
 
-    next = FbTk::StringUtil::getStringBetween(str,
-                                              line.c_str() + offset + next,
-                                              '{', '}');
+    next = getStringBetween(str, line.c_str() + offset + next, '{', '}');
 
     if (next <= 0) {
         cerr<<"Error parsing [startup] at column "<<offset<<" - expecting {command}."<<endl;
@@ -408,7 +405,7 @@ bool handleStartupItem(const string &line, int offset) {
     }
 
     // don't run command if fluxbox is restarting
-    if (Fluxbox::instance()->findScreen(screen)->isRestart())
+    if (fb->findScreen(screen)->isRestart())
         // the line was successfully read; we just didn't use it
         return true;
 
@@ -430,155 +427,146 @@ int parseApp(ifstream &file, Application &app, string *first_line = 0) {
     _FB_USES_NLS;
     int row = 0;
     while (! file.eof()) {
-        if (first_line || getline(file, line)) {
-            if (first_line) {
-                line = *first_line;
-                first_line = 0;
-            }
+        if (!(first_line || getline(file, line))) {
+            continue;
+        }
 
-            row++;
-            FbTk::StringUtil::removeFirstWhitespace(line);
-            FbTk::StringUtil::removeTrailingWhitespace(line);
-            if (line.size() == 0 || line[0] == '#')
-                continue;  //the line is commented or blank
+        if (first_line) {
+            line = *first_line;
+            first_line = 0;
+        }
 
-            int parse_pos = 0, err = 0;
-            string str_key, str_option, str_label;
+        row++;
+        if (isComment(line)) {
+            continue;
+        }
 
-            err = FbTk::StringUtil::getStringBetween(str_key,
-                                                     line.c_str(),
-                                                     '[', ']');
-            if (err > 0) {
-                int tmp;
-                tmp= FbTk::StringUtil::getStringBetween(str_option,
-                                                        line.c_str() + err,
-                                                        '(', ')');
-                if (tmp>0)
-                    err += tmp;
-            }
-            if (err > 0 ) {
-                parse_pos += err;
-                err = FbTk::StringUtil::getStringBetween(str_label,
-                                                         line.c_str() + parse_pos,
-                                                         '{', '}');
-                if (err>0) {
-                    parse_pos += err;
-                }
-            } else
-                continue; //read next line
+        string str_key, str_option, str_label;
+        int parse_pos = 0;
+        int err = getStringBetween(str_key, line.c_str(), '[', ']');
+        if (err > 0) {
+            int tmp = getStringBetween(str_option, line.c_str() + err, '(', ')');
+            if (tmp > 0)
+                err += tmp;
+        }
+        if (err > 0 ) {
+            parse_pos += err;
+            getStringBetween(str_label, line.c_str() + parse_pos, '{', '}');
+        } else
+            continue; //read next line
 
-            bool had_error = false;
+        bool had_error = false;
 
-            if (str_key.empty())
-                continue; //read next line
+        if (str_key.empty())
+            continue; //read next line
 
-            str_key = FbTk::StringUtil::toLower(str_key);
+        str_key = toLower(str_key);
+        str_label = toLower(str_label);
 
-            if (str_key == "workspace") {
-                unsigned int w;
-                if (FbTk::StringUtil::extractNumber(str_label, w))
-                    app.rememberWorkspace(w);
-                else
-                    had_error = true;
-            } else if (str_key == "head") {
-                unsigned int h;
-                if (FbTk::StringUtil::extractNumber(str_label, h))
-                    app.rememberHead(h);
-                else
-                    had_error = true;
-            } else if (str_key == "layer") {
-                int l = ResourceLayer::getNumFromString(str_label);
-                had_error = (l == -1);
-                if (!had_error)
-                    app.rememberLayer(l);
-            } else if (str_key == "dimensions") {
-                unsigned int h,w;
-                if (sscanf(str_label.c_str(), "%u %u", &w, &h) == 2) {
-                    app.rememberDimensions(w, h, false);
-                } else if(sscanf(str_label.c_str(), "%u%% %u%%", &w, &h) == 2) {
-                    app.rememberDimensions(w, h, true);
-                } else {
-                    had_error = true;
-                }
-            } else if (str_key == "position") {
-                FluxboxWindow::ReferenceCorner r = FluxboxWindow::LEFTTOP;
-                int x = 0, y = 0;
-                // more info about the parameter
-                // in ::rememberPosition
-
-                if (str_option.length())
-                    r = FluxboxWindow::getCorner(str_option);
-                had_error = (r == FluxboxWindow::ERROR);
-
-                if (!had_error){
-                    if(sscanf(str_label.c_str(), "%d %d", &x, &y) == 2) {
-                      app.rememberPosition(x, y, false, r);
-                    } else if (sscanf(str_label.c_str(), "%d%% %d%%", &x, &y) == 2){
-                      app.rememberPosition(x, y, true, r);
-                    }
-                } else {
-                    had_error = true;
-                }
-            } else if (str_key == "shaded") {
-                app.rememberShadedstate((strcasecmp(str_label.c_str(), "yes") == 0));
-            } else if (str_key == "tab") {
-                app.rememberTabstate((strcasecmp(str_label.c_str(), "yes") == 0));
-            } else if (str_key == "focushidden") {
-                app.rememberFocusHiddenstate((strcasecmp(str_label.c_str(), "yes") == 0));
-            } else if (str_key == "iconhidden") {
-                app.rememberIconHiddenstate((strcasecmp(str_label.c_str(), "yes") == 0));
-            } else if (str_key == "hidden") {
-                app.rememberIconHiddenstate((strcasecmp(str_label.c_str(), "yes") == 0));
-                app.rememberFocusHiddenstate((strcasecmp(str_label.c_str(), "yes") == 0));
-            } else if (str_key == "deco") {
-                int deco = WindowState::getDecoMaskFromString(str_label);
-                if (deco == -1)
-                    had_error = 1;
-                else
-                    app.rememberDecostate((unsigned int)deco);
-            } else if (str_key == "alpha") {
-                int focused_a, unfocused_a;
-                switch (sscanf(str_label.c_str(), "%i %i", &focused_a, &unfocused_a)) {
-                case 1: // 'alpha <focus>'
-                    unfocused_a = focused_a;
-                case 2: // 'alpha <focus> <unfocus>'
-                    focused_a = FbTk::Util::clamp(focused_a, 0, 255);
-                    unfocused_a = FbTk::Util::clamp(unfocused_a, 0, 255);
-                    app.rememberAlpha(focused_a, unfocused_a);
-                    break;
-                default:
-                    had_error = true;
-                    break;
-                }
-            } else if (str_key == "sticky") {
-                app.rememberStuckstate((strcasecmp(str_label.c_str(), "yes") == 0));
-            } else if (str_key == "focusnewwindow") {
-                app.rememberFocusNewWindow((strcasecmp(str_label.c_str(), "yes") == 0));
-            } else if (str_key == "minimized") {
-                app.rememberMinimizedstate((strcasecmp(str_label.c_str(), "yes") == 0));
-            } else if (str_key == "maximized") {
-                if (strcasecmp(str_label.c_str(), "yes") == 0)
-                    app.rememberMaximizedstate(WindowState::MAX_FULL);
-                else if (strcasecmp(str_label.c_str(), "horz") == 0)
-                    app.rememberMaximizedstate(WindowState::MAX_HORZ);
-                else if (strcasecmp(str_label.c_str(), "vert") == 0)
-                    app.rememberMaximizedstate(WindowState::MAX_VERT);
-                else
-                    app.rememberMaximizedstate(WindowState::MAX_NONE);
-            } else if (str_key == "fullscreen") {
-                app.rememberFullscreenstate((strcasecmp(str_label.c_str(), "yes") == 0));
-            } else if (str_key == "jump") {
-                app.rememberJumpworkspace((strcasecmp(str_label.c_str(), "yes") == 0));
-            } else if (str_key == "close") {
-                app.rememberSaveOnClose((strcasecmp(str_label.c_str(), "yes") == 0));
-            } else if (str_key == "end") {
-                return row;
+        if (str_key == "workspace") {
+            unsigned int w;
+            if (extractNumber(str_label, w))
+                app.rememberWorkspace(w);
+            else
+                had_error = true;
+        } else if (str_key == "head") {
+            unsigned int h;
+            if (extractNumber(str_label, h))
+                app.rememberHead(h);
+            else
+                had_error = true;
+        } else if (str_key == "layer") {
+            int l = ResourceLayer::getNumFromString(str_label);
+            had_error = (l == -1);
+            if (!had_error)
+                app.rememberLayer(l);
+        } else if (str_key == "dimensions") {
+            unsigned int h, w;
+            if (sscanf(str_label.c_str(), "%u %u", &w, &h) == 2) {
+                app.rememberDimensions(w, h, false);
+            } else if(sscanf(str_label.c_str(), "%u%% %u%%", &w, &h) == 2) {
+                app.rememberDimensions(w, h, true);
             } else {
-                cerr << _FB_CONSOLETEXT(Remember, Unknown, "Unknown apps key", "apps entry type not known")<<" = " << str_key << endl;
+                had_error = true;
             }
-            if (had_error) {
-                cerr<<"Error parsing apps entry: ("<<line<<")"<<endl;
+        } else if (str_key == "position") {
+            FluxboxWindow::ReferenceCorner r = FluxboxWindow::LEFTTOP;
+            int x = 0, y = 0;
+            // more info about the parameter
+            // in ::rememberPosition
+
+            if (str_option.length())
+                r = FluxboxWindow::getCorner(str_option);
+            had_error = (r == FluxboxWindow::ERROR);
+
+            if (!had_error){
+                if(sscanf(str_label.c_str(), "%d %d", &x, &y) == 2) {
+                  app.rememberPosition(x, y, false, r);
+                } else if (sscanf(str_label.c_str(), "%d%% %d%%", &x, &y) == 2){
+                  app.rememberPosition(x, y, true, r);
+                }
+            } else {
+                had_error = true;
             }
+        } else if (str_key == "shaded") {
+            app.rememberShadedstate(str_label == "yes");
+        } else if (str_key == "tab") {
+            app.rememberTabstate(str_label == "yes");
+        } else if (str_key == "focushidden") {
+            app.rememberFocusHiddenstate(str_label == "yes");
+        } else if (str_key == "iconhidden") {
+            app.rememberIconHiddenstate(str_label == "yes");
+        } else if (str_key == "hidden") {
+            app.rememberIconHiddenstate(str_label == "yes");
+            app.rememberFocusHiddenstate(str_label == "yes");
+        } else if (str_key == "deco") {
+            int deco = WindowState::getDecoMaskFromString(str_label);
+            if (deco == -1)
+                had_error = 1;
+            else
+                app.rememberDecostate((unsigned int)deco);
+        } else if (str_key == "alpha") {
+            int focused_a, unfocused_a;
+            switch (sscanf(str_label.c_str(), "%i %i", &focused_a, &unfocused_a)) {
+            case 1: // 'alpha <focus>'
+                unfocused_a = focused_a;
+            case 2: // 'alpha <focus> <unfocus>'
+                focused_a = FbTk::Util::clamp(focused_a, 0, 255);
+                unfocused_a = FbTk::Util::clamp(unfocused_a, 0, 255);
+                app.rememberAlpha(focused_a, unfocused_a);
+                break;
+            default:
+                had_error = true;
+                break;
+            }
+        } else if (str_key == "sticky") {
+            app.rememberStuckstate(str_label == "yes");
+        } else if (str_key == "focusnewwindow") {
+            app.rememberFocusNewWindow(str_label == "yes");
+        } else if (str_key == "minimized") {
+            app.rememberMinimizedstate(str_label == "yes");
+        } else if (str_key == "maximized") {
+            WindowState::MaximizeMode m = WindowState::MAX_NONE;
+            if (str_label == "yes")
+                m = WindowState::MAX_FULL;
+            else if (str_label == "horz")
+                m = WindowState::MAX_HORZ;
+            else if (str_label == "vert")
+                m = WindowState::MAX_VERT;
+            app.rememberMaximizedstate(m);
+        } else if (str_key == "fullscreen") {
+            app.rememberFullscreenstate(str_label == "yes");
+        } else if (str_key == "jump") {
+            app.rememberJumpworkspace(str_label == "yes");
+        } else if (str_key == "close") {
+            app.rememberSaveOnClose(str_label == "yes");
+        } else if (str_key == "end") {
+            return row;
+        } else {
+            cerr << _FB_CONSOLETEXT(Remember, Unknown, "Unknown apps key", "apps entry type not known")<<" = " << str_key << endl;
+        }
+        if (had_error) {
+            cerr<<"Error parsing apps entry: ("<<line<<")"<<endl;
         }
     }
     return row;
@@ -732,8 +720,10 @@ void Remember::checkReload() {
 }
 
 void Remember::reload() {
-    string apps_string = FbTk::StringUtil::expandFilename(Fluxbox::instance()->getAppsFilename());
 
+    Fluxbox& fb = *Fluxbox::instance();
+    string apps_string = expandFilename(fb.getAppsFilename());
+    bool ok = true;
 
     fbdbg<<"("<<__FUNCTION__<<"): Loading apps file ["<<apps_string<<"]"<<endl;
 
@@ -745,100 +735,98 @@ void Remember::reload() {
     m_pats.reset(new Patterns());
     m_startups.clear();
 
-    if (!apps_file.fail()) {
-        if (!apps_file.eof()) {
-            string line;
-            int row = 0;
-            bool in_group = false;
-            ClientPattern *pat = 0;
-            list<ClientPattern *> grouped_pats;
-            while (getline(apps_file, line) && ! apps_file.eof()) {
-                row++;
-                FbTk::StringUtil::removeFirstWhitespace(line);
-                FbTk::StringUtil::removeTrailingWhitespace(line);
-                if (line.size() == 0 || line[0] == '#')
-                    continue;
-                string key;
-                int err=0;
-                int pos = FbTk::StringUtil::getStringBetween(key,
-                                                             line.c_str(),
-                                                             '[', ']');
+    if (apps_file.fail()) {
+        ok = false;
+        cerr << "failed to open apps file " << apps_string << endl;
+    }
 
-                if (pos > 0 && (strcasecmp(key.c_str(), "app") == 0 ||
-                                strcasecmp(key.c_str(), "transient") == 0)) {
-                    ClientPattern *pat = new ClientPattern(line.c_str() + pos);
-                    if (!in_group) {
-                        if ((err = pat->error()) == 0) {
-                            bool transient = (strcasecmp(key.c_str(),
-                                                         "transient") == 0);
-                            Application *app = findMatchingPatterns(pat,
-                                                   old_pats, transient, false);
-                            if (app) {
-                                app->reset();
-                                reused_apps.insert(app);
-                            } else {
-                                app = new Application(transient, false);
-                            }
+    if (ok && apps_file.eof()) {
+        ok = false;
+        fbdbg<<"("<<__FUNCTION__<< ") Empty apps file" << endl;
+    }
 
-                            m_pats->push_back(make_pair(pat, app));
-                            row += parseApp(apps_file, *app);
-                        } else {
-                            cerr<<"Error reading apps file at line "<<row<<", column "<<(err+pos)<<"."<<endl;
-                            delete pat; // since it didn't work
-                        }
-                    } else {
-                        grouped_pats.push_back(pat);
-                    }
-                } else if (pos > 0 && strcasecmp(key.c_str(), "startup") == 0 &&
-                           Fluxbox::instance()->isStartup()) {
-                    if (!handleStartupItem(line, pos)) {
-                        cerr<<"Error reading apps file at line "<<row<<"."<<endl;
-                    }
-                    // save the item even if it was bad (aren't we nice)
-                    m_startups.push_back(line.substr(pos));
-                } else if (pos > 0 && strcasecmp(key.c_str(), "group") == 0) {
-                    in_group = true;
-                    if (line.find('(') != string::npos)
-                        pat = new ClientPattern(line.c_str() + pos);
-                } else if (in_group) {
-                    // otherwise assume that it is the start of the attributes
-                    Application *app = 0;
-                    // search for a matching app
-                    list<ClientPattern *>::iterator it = grouped_pats.begin();
-                    list<ClientPattern *>::iterator it_end = grouped_pats.end();
-                    while (!app && it != it_end) {
-                        app = findMatchingPatterns(*it, old_pats, false,
-                                                   in_group, pat);
-                        ++it;
-                    }
-
-                    if (!app)
-                        app = new Application(false, in_group, pat);
-                    else
-                        reused_apps.insert(app);
-
-                    while (!grouped_pats.empty()) {
-                        // associate all the patterns with this app
-                        m_pats->push_back(make_pair(grouped_pats.front(), app));
-                        grouped_pats.pop_front();
-                    }
-
-                    // we hit end... probably don't have attribs for the group
-                    // so finish it off with an empty application
-                    // otherwise parse the app
-                    if (!(pos>0 && strcasecmp(key.c_str(), "end") == 0)) {
-                        row += parseApp(apps_file, *app, &line);
-                    }
-                    in_group = false;
-                } else
-                    cerr<<"Error in apps file on line "<<row<<"."<<endl;
+    if (ok) {
+        string line;
+        int row = 0;
+        bool in_group = false;
+        ClientPattern *pat = 0;
+        list<ClientPattern *> grouped_pats;
+        while (getline(apps_file, line) && ! apps_file.eof()) {
+            row++;
 
+            if (isComment(line)) {
+                continue;
             }
-        } else {
-            fbdbg<<"("<<__FUNCTION__<< ") Empty apps file" << endl;
+
+            string key;
+            int err=0;
+            int pos = getStringBetween(key, line.c_str(), '[', ']');
+            string lc_key = toLower(key);
+
+            if (pos > 0 && (lc_key == "app" || lc_key == "transient")) {
+                ClientPattern *pat = new ClientPattern(line.c_str() + pos);
+                if (!in_group) {
+                    if ((err = pat->error()) == 0) {
+                        bool transient = (lc_key == "transient");
+                        Application *app = findMatchingPatterns(pat,
+                                               old_pats, transient, false);
+                        if (app) {
+                            app->reset();
+                            reused_apps.insert(app);
+                        } else {
+                            app = new Application(transient, false);
+                        }
+
+                        m_pats->push_back(make_pair(pat, app));
+                        row += parseApp(apps_file, *app);
+                    } else {
+                        cerr<<"Error reading apps file at line "<<row<<", column "<<(err+pos)<<"."<<endl;
+                        delete pat; // since it didn't work
+                    }
+                } else {
+                    grouped_pats.push_back(pat);
+                }
+            } else if (pos > 0 && lc_key == "startup" && fb.isStartup()) {
+                if (!handleStartupItem(line, pos)) {
+                    cerr<<"Error reading apps file at line "<<row<<"."<<endl;
+                }
+                // save the item even if it was bad (aren't we nice)
+                m_startups.push_back(line.substr(pos));
+            } else if (pos > 0 && lc_key == "group") {
+                in_group = true;
+                if (line.find('(') != string::npos)
+                    pat = new ClientPattern(line.c_str() + pos);
+            } else if (in_group) {
+                // otherwise assume that it is the start of the attributes
+                Application *app = 0;
+                // search for a matching app
+                list<ClientPattern *>::iterator it = grouped_pats.begin();
+                list<ClientPattern *>::iterator it_end = grouped_pats.end();
+                for (; !app && it != it_end; ++it) {
+                    app = findMatchingPatterns(*it, old_pats, false, in_group, pat);
+                }
+
+                if (!app)
+                    app = new Application(false, in_group, pat);
+                else
+                    reused_apps.insert(app);
+
+                while (!grouped_pats.empty()) {
+                    // associate all the patterns with this app
+                    m_pats->push_back(make_pair(grouped_pats.front(), app));
+                    grouped_pats.pop_front();
+                }
+
+                // we hit end... probably don't have attribs for the group
+                // so finish it off with an empty application
+                // otherwise parse the app
+                if (!(pos>0 && lc_key == "end")) {
+                    row += parseApp(apps_file, *app, &line);
+                }
+                in_group = false;
+            } else
+                cerr<<"Error in apps file on line "<<row<<"."<<endl;
         }
-    } else {
-        cerr << "failed to open apps file" << endl;
     }
 
     // Clean up old state
@@ -994,7 +982,7 @@ void Remember::save() {
             case (WindowState::DECOR_BORDER):
                 apps_file << "  [Deco]\t{BORDER}" << endl;
                 break;
-            case (WindowState::DECORM_TAB):
+            case (WindowState::DECOR_TAB):
                 apps_file << "  [Deco]\t{TAB}" << endl;
                 break;
             default:
@@ -1136,7 +1124,7 @@ void Remember::rememberAttrib(WinClient &winclient, Attribute attrib) {
         app = add(winclient);
         if (!app) return;
     }
-    int head, head_x, head_y, win_w, win_h, percx, percy;
+    int head, percx, percy;
     switch (attrib) {
     case REM_WORKSPACE:
         app->rememberWorkspace(win->workspaceNumber());
@@ -1146,15 +1134,15 @@ void Remember::rememberAttrib(WinClient &winclient, Attribute attrib) {
         break;
     case REM_DIMENSIONS: {
         head = win->screen().getHead(win->fbWindow());
-        int percx = win->screen().calRelativeDimensionWidth(head, win->normalWidth());
-        int percy = win->screen().calRelativeDimensionHeight(head, win->normalHeight());
+        percx = win->screen().calRelativeDimensionWidth(head, win->normalWidth());
+        percy = win->screen().calRelativeDimensionHeight(head, win->normalHeight());
         app->rememberDimensions(percx, percy, true);
         break;
     }
     case REM_POSITION: {
         head = win->screen().getHead(win->fbWindow());
-        int percx = win->screen().calRelativePositionWidth(head, win->normalX());
-        int percy = win->screen().calRelativePositionHeight(head, win->normalY());
+        percx = win->screen().calRelativePositionWidth(head, win->normalX());
+        percy = win->screen().calRelativePositionHeight(head, win->normalY());
         app->rememberPosition(percx, percy, true);
         break;
     }
@@ -1465,10 +1453,10 @@ void Remember::updateClientClose(WinClient &winclient) {
 
 }
 
-void Remember::initForScreen(BScreen &screen) {
-    // All windows get the remember menu.
-    _FB_USES_NLS;
-    screen.addExtraWindowMenu(_FB_XTEXT(Remember, MenuItemName, "Remember...", "Remember item in menu"),
-                              createRememberMenu(screen));
+void Remember::initForScreen(BScreen &screen) { }
 
+FbTk::Menu* Remember::createMenu(BScreen& screen) {
+
+    return createRememberMenu(screen);
 }
+
