@@ -24,15 +24,6 @@
 
 #include "Slit.hh"
 
-//use GNU extensions
-#ifndef	 _GNU_SOURCE
-#define	 _GNU_SOURCE
-#endif // _GNU_SOURCE
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif // HAVE_CONFIG_H
-
 #include "Screen.hh"
 #include "ScreenPlacement.hh"
 #include "FbTk/ImageControl.hh"
@@ -58,17 +49,13 @@
 
 #include "FbTk/App.hh"
 #include "FbTk/MenuSeparator.hh"
+#include "FbTk/FileUtil.hh"
 #include "FbTk/StringUtil.hh"
 #include "FbTk/I18n.hh"
 #include "FbTk/BoolMenuItem.hh"
 #include "FbTk/IntMenuItem.hh"
 #include "FbTk/RadioMenuItem.hh"
 
-#ifdef HAVE_SYS_STAT_H
-#include <sys/types.h>
-#include <sys/stat.h>
-#endif // HAVE_SYS_STAT_H
-
 #include <X11/Xatom.h>
 
 #include <iostream>
@@ -262,6 +249,8 @@ Slit::Slit(BScreen &scr, FbTk::Layer &layer, const char *filename)
 
     _FB_USES_NLS;
 
+    frame.window.setWindowRole("fluxbox-slit");
+
     // attach to theme and root window change signal
     join(theme().reconfigSig(), FbTk::MemFun(*this, &Slit::reconfigure));
 
@@ -279,7 +268,7 @@ Slit::Slit(BScreen &scr, FbTk::Layer &layer, const char *filename)
     // move the frame out of sight for a moment
     frame.window.move(-frame.window.width(), -frame.window.height());
     // setup timer
-    m_timer.setTimeout(200); // default timeout
+    m_timer.setTimeout(200L * FbTk::FbTime::IN_MILLISECONDS); // default timeout
     m_timer.fireOnce(true);
     FbTk::RefCount<FbTk::Command<void> > toggle_hidden(new FbTk::SimpleCommand<Slit>(*this, &Slit::toggleHidden));
     m_timer.setCommand(toggle_hidden);
@@ -879,7 +868,7 @@ void Slit::clientUp(SlitClient* client) {
     }
 
     SlitClients::iterator it = m_client_list.begin();
-    for(it++; it != m_client_list.end(); it++) {
+    for(++it; it != m_client_list.end(); ++it) {
         if ((*it) == client) {
             SlitClients::iterator prev = it;
             prev--;
@@ -900,7 +889,7 @@ void Slit::clientDown(SlitClient* client) {
     }
 
     SlitClients::reverse_iterator it = m_client_list.rbegin();
-    for(it++; it != m_client_list.rend(); it++) {
+    for(++it; it != m_client_list.rend(); ++it) {
         if ((*it) == client) {
             SlitClients::reverse_iterator next = it;
             next--;
@@ -1050,10 +1039,12 @@ void Slit::toggleHidden() {
     if (doAutoHide()) {
         if (!m_slitmenu.isVisible()) {
             m_timer.fireOnce(true);
-        } else
+        } else {
             return;
-    } else
-        if (!isHidden()) return;
+        }
+    //} else if (!isHidden()) {
+    //    return;
+    }
 
     m_hidden = ! m_hidden; // toggle hidden state
     if (isHidden())
@@ -1070,29 +1061,30 @@ void Slit::loadClientList(const char *filename) {
     m_filename = filename;
     string real_filename= FbTk::StringUtil::expandFilename(filename);
 
-    struct stat buf;
-    if (stat(real_filename.c_str(), &buf) == 0) {
-        ifstream file(real_filename.c_str());
-        string name;
-        while (! file.eof()) {
-            name = "";
-            getline(file, name); // get the entire line
-            if (name.empty())
-                continue;
+    if (!FbTk::FileUtil::isRegularFile(real_filename.c_str())) {
+        return;
+    }
 
-            // remove whitespaces from start and end
-            FbTk::StringUtil::removeFirstWhitespace(name);
+    ifstream file(real_filename.c_str());
+    string name;
+    while (! file.eof()) {
+        name.clear();
+        getline(file, name); // get the entire line
+        if (name.empty())
+            continue;
 
-            // the cleaned string could still be a comment, or blank
-            if ( name.empty() || name[0] == '#' || name[0] == '!' )
-                continue;
+        // remove whitespaces from start and end
+        FbTk::StringUtil::removeFirstWhitespace(name);
 
-            // trailing whitespace won't affect the above test
-            FbTk::StringUtil::removeTrailingWhitespace(name);
+        // the cleaned string could still be a comment, or blank
+        if ( name.empty() || name[0] == '#' || name[0] == '!' )
+            continue;
 
-            SlitClient *client = new SlitClient(name.c_str());
-            m_client_list.push_back(client);
-        }
+        // trailing whitespace won't affect the above test
+        FbTk::StringUtil::removeTrailingWhitespace(name);
+
+        SlitClient *client = new SlitClient(name.c_str());
+        m_client_list.push_back(client);
     }
 }
 
@@ -1107,21 +1099,21 @@ void Slit::updateClientmenu() {
 
     FbTk::RefCount<FbTk::Command<void> > cycle_up(new FbTk::SimpleCommand<Slit>(*this, &Slit::cycleClientsUp));
     FbTk::RefCount<FbTk::Command<void> > cycle_down(new FbTk::SimpleCommand<Slit>(*this, &Slit::cycleClientsDown));
-    m_clientlist_menu.insert(_FB_XTEXT(Slit, CycleUp, "Cycle Up", "Cycle clients upwards"), cycle_up);
-    m_clientlist_menu.insert(_FB_XTEXT(Slit, CycleDown, "Cycle Down", "Cycle clients downwards"), cycle_down);
+    m_clientlist_menu.insertCommand(_FB_XTEXT(Slit, CycleUp, "Cycle Up", "Cycle clients upwards"), cycle_up);
+    m_clientlist_menu.insertCommand(_FB_XTEXT(Slit, CycleDown, "Cycle Down", "Cycle clients downwards"), cycle_down);
 
-    m_clientlist_menu.insert(new FbTk::MenuSeparator());
+    m_clientlist_menu.insertItem(new FbTk::MenuSeparator());
 
     FbTk::RefCount<FbTk::Command<void> > reconfig(new FbTk::SimpleCommand<Slit>(*this, &Slit::reconfigure));
     SlitClients::iterator it = m_client_list.begin();
     for (; it != m_client_list.end(); ++it) {
         if ((*it) != 0 && (*it)->window() != 0)
-            m_clientlist_menu.insert(new SlitClientMenuItem(*this, *(*it), reconfig));
+            m_clientlist_menu.insertItem(new SlitClientMenuItem(*this, *(*it), reconfig));
     }
 
-    m_clientlist_menu.insert(new FbTk::MenuSeparator());
+    m_clientlist_menu.insertItem(new FbTk::MenuSeparator());
     FbTk::RefCount<FbTk::Command<void> > savecmd(new FbTk::SimpleCommand<Slit>(*this, &Slit::saveClientList));
-    m_clientlist_menu.insert(_FB_XTEXT(Slit,
+    m_clientlist_menu.insertCommand(_FB_XTEXT(Slit,
                                      SaveSlitList,
                                      "Save SlitList", "Saves the current order in the slit"),
                              savecmd);
@@ -1173,31 +1165,32 @@ void Slit::setupMenu() {
 
     // setup base menu
     m_slitmenu.setLabel(_FB_XTEXT(Slit, Slit, "Slit", "The Slit"));
-    m_slitmenu.insert(_FB_XTEXT(Menu, Placement, "Placement", "Title of Placement menu"),
-                      placement_menu);
+    m_slitmenu.insertSubmenu(
+        _FB_XTEXT(Menu, Placement, "Placement", "Title of Placement menu"),
+        placement_menu);
 
-    m_slitmenu.insert(_FB_XTEXT(Menu, Layer, "Layer...", "Title of Layer menu"), m_layermenu.get());
+    m_slitmenu.insertSubmenu(
+        _FB_XTEXT(Menu, Layer, "Layer...", "Title of Layer menu"),
+        m_layermenu.get());
 
 #ifdef XINERAMA
     if (screen().hasXinerama()) {
-        m_slitmenu.insert(_FB_XTEXT(Menu, OnHead, "On Head...", "Title of On Head menu"),
-                          m_xineramaheadmenu =
-                          new XineramaHeadMenu<Slit>(
-                              screen().menuTheme(),
-                              screen(),
-                              screen().imageControl(),
-                              *screen().layerManager().getLayer(::ResourceLayer::MENU),
-                              *this,
-                              _FB_XTEXT(Slit, OnHead, "Slit on Head", "Title of Slits On Head menu")
-                              ));
+        m_xineramaheadmenu = new XineramaHeadMenu<Slit>(
+            screen().menuTheme(),
+            screen(),
+            screen().imageControl(),
+            *screen().layerManager().getLayer(::ResourceLayer::MENU),
+            *this,
+            _FB_XTEXT(Slit, OnHead, "Slit on Head", "Title of Slits On Head menu"));
+        m_slitmenu.insertSubmenu(_FB_XTEXT(Menu, OnHead, "On Head...", "Title of On Head menu"), m_xineramaheadmenu);
     }
 #endif //XINERAMA
 
-    m_slitmenu.insert(new FbTk::BoolMenuItem(_FB_XTEXT(Common, AutoHide, "Auto hide", "This thing automatically hides when not close by"),
+    m_slitmenu.insertItem(new FbTk::BoolMenuItem(_FB_XTEXT(Common, AutoHide, "Auto hide", "This thing automatically hides when not close by"),
                                        m_rc_auto_hide,
                                        save_and_reconfigure_slit));
 
-    m_slitmenu.insert(new FbTk::BoolMenuItem(_FB_XTEXT(Common, MaximizeOver,"Maximize Over", "Maximize over this thing when maximizing"),
+    m_slitmenu.insertItem(new FbTk::BoolMenuItem(_FB_XTEXT(Common, MaximizeOver,"Maximize Over", "Maximize over this thing when maximizing"),
                                        m_rc_maximize_over,
                                        save_and_reconfigure_slit));
 
@@ -1214,9 +1207,9 @@ void Slit::setupMenu() {
     RefCount<Command<void> > set_alpha_cmd(alpha_macrocmd);
     alpha_menuitem->setCommand(set_alpha_cmd);
 
-    m_slitmenu.insert(alpha_menuitem);
+    m_slitmenu.insertItem(alpha_menuitem);
 
-    m_slitmenu.insert(_FB_XTEXT(Slit, ClientsMenu, "Clients", "Slit client menu"), &m_clientlist_menu);
+    m_slitmenu.insertSubmenu(_FB_XTEXT(Slit, ClientsMenu, "Clients", "Slit client menu"), &m_clientlist_menu);
     m_slitmenu.updateMenu();
 
     // setup sub menu
@@ -1257,10 +1250,8 @@ void Slit::setupMenu() {
             placement_menu->insert("");
             placement_menu->setItemEnabled(i, false);
         } else {
-            placement_menu->insert(new PlaceSlitMenuItem(str, *this,
-                                                          placement,
-                                                          save_and_reconfigure));
-
+            FbTk::MenuItem* item = new PlaceSlitMenuItem(str, *this, placement, save_and_reconfigure);
+            placement_menu->insertItem(item);
         }
         place_menu.pop_front();
     }
@@ -1276,7 +1267,6 @@ void Slit::moveToLayer(int layernum) {
 
 void Slit::saveOnHead(int head) {
     m_rc_on_head = head;
-    // reposition
     reconfigure();
 }
 
