@@ -20,60 +20,69 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Xutil.cc 4199 2006-02-16 06:53:05Z mathias $
-
 #include "Xutil.hh"
+#include "Debug.hh"
 
 #include "FbTk/I18n.hh"
 #include "FbTk/App.hh"
 
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
-#include <X11/Xlib.h>
 #include <iostream>
-using namespace std;
+
+#ifdef HAVE_CSTRING
+  #include <cstring>
+#else
+  #include <string.h>
+#endif
+
+using std::string;
+using std::strlen;
+using std::endl;
+
 
 namespace Xutil {
 
-std::string getWMName(Window window) {
+FbTk::FbString getWMName(Window window) {
 
     if (window == None)
-        return "";
+        return FbTk::FbString("");
 
     Display *display = FbTk::App::instance()->display();
 
     XTextProperty text_prop;
     text_prop.value = 0;
-    char **list;
-    int num;
+    char **list = 0;
+    int num = 0;
     _FB_USES_NLS;
-    std::string name;
+    FbTk::FbString name;
 
     if (XGetWMName(display, window, &text_prop)) {
         if (text_prop.value && text_prop.nitems > 0) {
             if (text_prop.encoding != XA_STRING) {
-				
+
                 text_prop.nitems = strlen((char *) text_prop.value);
-				
-                if ((XmbTextPropertyToTextList(display, &text_prop,
-                                               &list, &num) == Success) &&
-                    (num > 0) && *list) {
-                    name = static_cast<char *>(*list);
+                XmbTextPropertyToTextList(display, &text_prop, &list, &num);
+
+                if (num > 0 && list != 0)
+                    name = FbTk::FbStringUtil::LocaleStrToFb(static_cast<char *>(*list));
+                else
+                    name = text_prop.value ? FbTk::FbStringUtil::XStrToFb((char *)text_prop.value) : "";
+
+                if (list)
                     XFreeStringList(list);
-                } else
-                    name = text_prop.value ? (char *)text_prop.value : "";
-					
-            } else				
-                name = text_prop.value ? (char *)text_prop.value : "";
+
+            } else
+                name = text_prop.value ? FbTk::FbStringUtil::XStrToFb((char *)text_prop.value) : "";
 
             XFree(text_prop.value);
 
         } else { // default name
-            name = _FBTEXT(Window, Unnamed, "Unnamed", "Default name for a window without a WM_NAME");
+            name = _FB_XTEXT(Window, Unnamed, "Unnamed", "Default name for a window without a WM_NAME");
         }
     } else {
         // default name
-        name = _FBTEXT(Window, Unnamed, "Unnamed", "Default name for a window without a WM_NAME");
+        name = _FB_XTEXT(Window, Unnamed, "Unnamed", "Default name for a window without a WM_NAME");
     }
 
     return name;
@@ -81,55 +90,48 @@ std::string getWMName(Window window) {
 
 
 // The name of this particular instance
-std::string getWMClassName(Window win) {
+FbTk::FbString getWMClassName(Window win) {
+
     XClassHint ch;
-    std::string instance_name;
+    FbTk::FbString instance_name;
 
     if (XGetClassHint(FbTk::App::instance()->display(), win, &ch) == 0) {
-#ifdef DEBUG
-        cerr<<"Xutil: Failed to read class hint!"<<endl;
-#endif //DEBUG
-        instance_name = "";
-    } else {        
+        fbdbg<<"Xutil: Failed to read class hint!"<<endl;
+    } else {
 
         XFree(ch.res_class);
-        
-        if (ch.res_class != 0) {
+
+        if (ch.res_name != 0) {
             instance_name = const_cast<char *>(ch.res_name);
             XFree(ch.res_name);
             ch.res_name = 0;
-        } else
-            instance_name = "";
+        }
     }
 
     return instance_name;
-
 }
 
 // the name of the general class of the app
-std::string getWMClassClass(Window win) {
+FbTk::FbString getWMClassClass(Window win) {
+
     XClassHint ch;
-    std::string class_name;
+    FbTk::FbString class_name;
 
     if (XGetClassHint(FbTk::App::instance()->display(), win, &ch) == 0) {
-#ifdef DEBUG
-        cerr<<"Xutil: Failed to read class hint!"<<endl;
-#endif //DEBUG
-        class_name = "";
-    } else {        
+        fbdbg<<"Xutil: Failed to read class hint!"<<endl;
+    } else {
 
         XFree(ch.res_name);
-        
+
         if (ch.res_class != 0) {
             class_name = const_cast<char *>(ch.res_class);
             XFree(ch.res_class);
             ch.res_class = 0;
-        } else
-            class_name = "";
+        }
     }
 
     return class_name;
 }
 
-}; // end namespace Xutil
+} // end namespace Xutil
 
